@@ -9,7 +9,7 @@ use Carp;
 
 use Math::Trig qw(atan asin acos tan);
 
-our $VERSION = '0.1.3';
+our $VERSION = '0.2.0';
 
 =head1 NAME
 
@@ -18,7 +18,7 @@ Math::Expression::Evaluator - parses, compiles and evaluates mathematic expressi
 =head1 SYNOPSIS
 
     use Math::Expression::Evaluator;
-    my $m = new Math::Expression::Evaluator;
+    my $m = Math::Expression::Evaluator->new;
 
     print $m->parse("a = 12; a*3")->val(), "\n";
     # prints 36
@@ -122,6 +122,20 @@ Returns a reference to the object, so that method calls can be chained:
 
 Parse failures cause this method to die with a stack trace. 
 
+You can call C<parse> on an existing Math::Expression::Evaluator object to
+re-use it, in which case previously set variables and callbacks persist
+between calls.
+
+This (perhaps contrived) example explains this:
+
+    my $m = Math::Expression::Evaluator->new('a = 3; a');
+    $m->val();
+    $m->parse('a + 5');
+    print $m->val(), "\n"   # prints 8, because a = 3 was re-used
+
+If that's not what you want, create a new object instead - the constructor is
+rather cheap.
+
 =item compiled
 
 Returns an anonymous function that is a compiled version of the current
@@ -142,7 +156,7 @@ Executes the AST generated by parse(), and returns the number that the
 expression is evaluated to. It accepts an optional hash reference that
 contain values for variables:
 
-    my $m = new MathExpr;
+    my $m = MathExpr->new;
     $m->parse("(x - 1) / (x + 1)");
     foreach (0 .. 10) {
         print $_, "\t", $m->val({x => $_}), "\n";
@@ -161,6 +175,24 @@ multiple times.
 
 C<variables()> returns a list of variables that are used in the expression.
 
+=item set_var_callback
+
+Allows you to set a callback which the Match::Expression::Evaluator object
+calls when it can't find a variable. The name of the variable is passed in
+as the first argument. If the callback function can't handle that variable
+either, it should die, just like the default one does.
+
+    my $m = Math::Expression::Evaluator->new();
+    $m->parse('1 + a');
+    my $callback = sub { ord($_[0]) };
+    $m->set_var_callback($callback);
+    print $m->val();    # calls $callback, which returns 97
+                        # so $m->val() return 98
+
+The callback will be called every time the variable is accessed, so if it
+requires expensive calculations, you are encouraged to cache it either
+yourself our automatically with L<Memoize>.
+
 =item ast_size
 
 C<ast_size> returns an integer which gives a crude measure of the logical
@@ -245,9 +277,10 @@ a commit bit to the github repository, I'm giving them out freely.
 sub new {
     my $class = shift;
     my $self = bless {}, $class;
-    $self->{tokens} = [];
-    $self->{variables} = {};
-    
+    $self->{tokens}       = [];
+    $self->{variables}    = {};
+    $self->{var_callback} = sub { confess "Variable '$_[0]' not defined" };
+
     my $first = shift;
 
     if (defined $first){
@@ -266,7 +299,7 @@ sub new {
 # parse a text into an AST, stores the AST in $self->{ast}
 sub parse {
     my ($self, $text) = @_;
-    $self->{ast} = 
+    $self->{ast} =
         Math::Expression::Evaluator::Parser::parse($text, $self->{config});
     return $self;
 }
@@ -309,6 +342,10 @@ sub _execute {
     }
 }
 
+sub set_var_callback {
+    $_[0]->{var_callback} = $_[1];
+}
+
 # executes a sum
 sub _exec_sum {
     my $self = shift;
@@ -400,7 +437,7 @@ sub _variable_lookup {
     } elsif (exists $self->{variables}->{$var}){
         return $self->{variables}->{$var};
     } else {
-        confess("Variable '$var' not defined");
+        $self->{var_callback}->($var);
     }
 }
 
@@ -444,12 +481,12 @@ sub _ast_to_perl {
     my $joined_operator = sub {
         my $op = shift;
         return sub {
-            join $op, map { '(' . _ast_to_perl($_).  ')' } @_ 
+            join $op, map { '(' . _ast_to_perl($_).  ')' } @_
         };
     };
 
     my %translations = (
-        '$'     => sub { qq/( exists \$vars{$_[0]} ? \$vars{$_[0]} : \$default_vars{$_[0]})/ },
+        '$'     => sub { qq/( exists \$vars{$_[0]} ? \$vars{$_[0]} : exists \$default_vars{$_[0]} ? \$default_vars{$_[0]} : \$self->{var_callback}->("$_[0]")) / },
         '{'     => sub { join "\n", map { _ast_to_perl($_) . ";" } @_ },
         '='     => sub { qq/\$vars{$_[0][1]} = / . _ast_to_perl($_[1]) },
         '+'     => &$joined_operator('+'),
